@@ -77,7 +77,11 @@ class SimpleTree:
                 print field_name
             return
         else:
+            #Check if this is an update and if this an update find the previous simulations list:
+            prev_simulations_list=self.load_previous_simulations(options)
+
             simulations_list=discover.discover_simulations_recursive(self,options,self.drs.simulations_desc)
+            simulations_list=sorted(list(set(simulations_list).difference(prev_simulations_list)))
             print "This is a list of simulations that COULD satisfy the query:"
             for simulation in simulations_list:
                 print ','.join(simulation)
@@ -101,6 +105,26 @@ class SimpleTree:
             output.close()
         return
 
+    def load_previous_simulations(self,options):
+        prev_simulations_list=[]
+        if ('update' in dir(options) and
+             getattr(options,'update')!=None):
+             for update_file in options.update:
+                 #There is an update file:
+                 options_copy=copy.copy(options)
+                 options_copy.in_diagnostic_netcdf_file=update_file
+                 #Load the header to update:
+                 self.define_database(options_copy)
+                 old_header=self.nc_Database.load_header()
+                 self.close_database()
+                 if remove_entry_from_dictionary(self.header,'search_list')==remove_entry_from_dictionary(old_header,'search_list'):
+                    prev_simulations_list.extend(self.list_fields_local(options_copy,self.drs.simulations_desc))
+        if len(prev_simulations_list)>0:
+            print 'Updating, not considering the following simulations:'
+            for simulation in prev_simulations_list:
+                print ','.join(simulation)
+        return sorted(prev_simulations_list)
+
     def validate(self,options):
         self.load_header(options)
         #if options.data_nodes!=None:
@@ -173,7 +197,7 @@ class SimpleTree:
         self.load_database(options,find_simple)
 
         #Find data node list:
-        data_node_list=self.nc_Database.list_data_nodes(options)
+        data_node_list=[item[0] for item in self.nc_Database.list_fields(['data_node'])]
         paths_list=self.nc_Database.list_paths()
         self.close_database()
 
@@ -191,11 +215,11 @@ class SimpleTree:
                 print 'Using min year {0} for experiment {1}'.format(str(min_year),experiment)
 
         #Start the retrieval workers:
-        #if not ('serial' in dir(options) and options.serial):
-        #    processes=dict()
-        #    for data_node in data_node_list:
-        #        processes[data_node]=multiprocessing.Process(target=worker_retrieve, args=(queues[data_node], queues['end']))
-        #        processes[data_node].start()
+        if not ('serial' in dir(options) and options.serial):
+            processes=dict()
+            for data_node in data_node_list:
+                processes[data_node]=multiprocessing.Process(target=worker_retrieve, args=(queues[data_node], queues['end']))
+                processes[data_node].start()
 
         #Find the data that needs to be recovered:
         self.define_database(options)
@@ -211,7 +235,7 @@ class SimpleTree:
         self.load_database(options,find_simple)
         simulations_list=self.nc_Database.list_fields(self.drs.simulations_desc)
 
-        data_node_list=self.nc_Database.list_data_nodes(options)
+        data_node_list=[item[0] for item in self.nc_Database.list_fields(['data_node'])]
         url_list=[self.nc_Database.list_paths_by_data_node(data_node).split('|')[0].replace('fileServer','dodsC')
                     for data_node in data_node_list ]
         self.close_database()
@@ -256,9 +280,24 @@ class SimpleTree:
         return
 
     def load_header(self,options):
-        self.define_database(options)
-        self.header=self.nc_Database.load_header()
-        self.close_database()
+        if ('in_diagnostic_headers_file' in dir(options) and 
+           options.in_diagnostic_headers_file!=None):
+            try:
+                self.header=json.load(open(options.in_diagnostic_headers_file,'r'))['header']
+            except ValueError as e:
+                print 'The input diagnostic file '+options.in_diagnostic_headers_file+' does not conform to JSON standard. Make sure to check its syntax'
+            #for field_to_limit in ['experiment_list','variable_list']:
+            #    if (field_to_limit in dir(options) and
+            #        getattr(options,field_to_limit)!=None):
+            #        setattr(options,field_to_limit,
+            #                list(set(getattr(option,field_to_limit)).intersection(self.header[field_to_limit].keys())))
+            #    else:
+            #        setattr(options,field_to_limit,
+            #                self.header[field_to_limit].keys())
+        else:
+            self.define_database(options)
+            self.header=self.nc_Database.load_header()
+            self.close_database()
         return
 
     def record_header(self,options,output):
@@ -363,7 +402,7 @@ def launch_download_and_remote_retrieve(output,data_node_list,queues,retrieval_f
     #Second step: Process the queues:
     #print datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
     start_time = datetime.datetime.now()
-    print('Retrieving from data nodes:')
+    print('Remaining retrieval from data nodes:')
     queues_size=dict()
     for data_node in data_node_list:
         queues_size[data_node]=queues[data_node].qsize()
@@ -380,13 +419,13 @@ def launch_download_and_remote_retrieve(output,data_node_list,queues,retrieval_f
                 progress_report(retrieval_function,output,tuple,queues,queues_size,data_node_list,start_time)
         
     else:
-        #for data_node in data_node_list:
-        #    queues[data_node].put('STOP')
-        processes=dict()
         for data_node in data_node_list:
             queues[data_node].put('STOP')
-            processes[data_node]=multiprocessing.Process(target=worker_retrieve, args=(queues[data_node], queues['end']))
-            processes[data_node].start()
+        #processes=dict()
+        #for data_node in data_node_list:
+        #    queues[data_node].put('STOP')
+        #    processes[data_node]=multiprocessing.Process(target=worker_retrieve, args=(queues[data_node], queues['end']))
+        #    processes[data_node].start()
 
         for data_node in data_node_list:
             for tuple in iter(queues['end'].get, 'STOP'):
@@ -415,6 +454,7 @@ def progress_report(retrieval_function,output,tuple,queues,queues_size,data_node
                             data_node in data_node_list]
         elapsed_time = datetime.datetime.now() - start_time
         print str(elapsed_time)+', '+' | '.join(string_to_print)+'\r',
+        #print str(elapsed_time)+'\r'
         #for data_node in data_node_list:
         #    print data_node, queues[data_node].qsize()
     return
@@ -438,3 +478,6 @@ def define_queues(options,data_node_list):
     if 'source_dir' in dir(options) and options.source_dir!=None:
         queues[retrieval_utils.get_data_node(options.source_dir,'local_file')]=multiprocessing.Queue()
     return queues
+
+def remove_entry_from_dictionary(dictio,entry):
+    return {name:dictio[name] for name in dictio.keys() if name!=entry}
