@@ -7,10 +7,6 @@ def input_arguments_json(parser):
     parser.add_argument('in_diagnostic_headers_file',
                                  help='Diagnostic headers file (input)')
     return
-def output_arguments_json(parser):
-    parser.add_argument('out_diagnostic_headers_file',
-                                 help='Diagnostic headers file (output)')
-    return
 
 def input_arguments(parser):
     parser.add_argument('in_diagnostic_netcdf_file',
@@ -88,6 +84,9 @@ def ask(subparsers,epilog,project_drs):
                                  help='When this option is activated, queried experiments are assumed to be related.\n\
                                        In this situation, cdb_query will discard ({0}) tuples that do not have variables for\n\
                                        ALL of the requested experiments'.format(','.join(project_drs.simulations_desc)))
+    parser.add_argument('--update',
+                                 type=str,action='append',
+                                 help='Update the specified file. Will only ask for simulations that were not previously found.')
     input_arguments_json(parser)
     output_arguments(parser)
     parser.add_argument('--list_only_field',default=None, choices=project_drs.remote_fields,
@@ -158,6 +157,9 @@ def validate(subparsers,epilog,project_drs):
 
     input_arguments(parser)
     output_arguments(parser)
+    parser.add_argument('--in_diagnostic_headers_file',
+                                 help='Alternative diagnostic headers file (to modify target validate)',\
+                                 type=str,default=None)
     proc_group = parser.add_argument_group('These arguments set threading options')
     proc_group.add_argument('--num_procs',
                                  default=1, type=int,#choices=xrange(1,6),
@@ -200,24 +202,24 @@ def download(subparsers,epilog,project_drs):
 
     inc_group = parser.add_argument_group('Inclusions')
     inc_group.add_argument('--year',
-                                 default=None, type=int, nargs='*',
-                                 help='Retrieve only this year.')
+                                 default=None, type=int_list,
+                                 help='Retrieve only these comma-separated years.')
     inc_group.add_argument('--month',
-                                 default=None, type=int, nargs='*',
-                                 help='Retrieve only this month (1 to 12).')
+                                 default=None, type=int_list,
+                                 help='Retrieve only these comma-separated months (1 to 12).')
     inc_group.add_argument('--day',
-                                 default=None, type=int, nargs='*',
-                                 help='Retrieve only this calendar day.')
+                                 default=None, type=int_list,
+                                 help='Retrieve only these comma-separated calendar days.')
     #slicing_arguments(inc_group,project_drs)
     #exc_group = parser.add_argument_group('Exclusions')
     #excluded_slicing_arguments(exc_group,project_drs)
     slicing_arguments(inc_group,project_drs,action_type='append')
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs,action_type='append')
-    comp_group = parser.add_argument_group('Complex Query')
-    comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
-                                       help='List the field (or fields if repeated) found in the file' )
-    complex_slicing(comp_group,project_drs,action_type='append')
+    #comp_group = parser.add_argument_group('Complex Query')
+    #comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
+    #                                   help='List the field (or fields if repeated) found in the file' )
+    #complex_slicing(comp_group,project_drs,action_type='append')
 
     data_node_group = parser.add_argument_group('Limit download from specific data nodes')
     data_node_group.add_argument('--data_node',type=str,action='append',help='Retrieve only from the specified data nodes')
@@ -250,24 +252,24 @@ def download_raw(subparsers,epilog,project_drs):
 
     inc_group = parser.add_argument_group('Inclusions')
     inc_group.add_argument('--year',
-                                 default=None, type=int, nargs='*',
-                                 help='Retrieve only this year.')
+                                 default=None, type=int_list,
+                                 help='Retrieve only these comma-separated years.')
     inc_group.add_argument('--month',
-                                 default=None, type=int, nargs='*',
-                                 help='Retrieve only this month (1 to 12).')
+                                 default=None, type=int_list,
+                                 help='Retrieve only these comma-separated months (1 to 12).')
     inc_group.add_argument('--day',
-                                 default=None, type=int, nargs='*',
-                                 help='Retrieve only this calendar day.')
+                                 default=None, type=int_list,
+                                 help='Retrieve only these comma-separated calendar days.')
     #slicing_arguments(inc_group,project_drs)
     #exc_group = parser.add_argument_group('Exclusions')
     #excluded_slicing_arguments(exc_group,project_drs)
     slicing_arguments(inc_group,project_drs,action_type='append')
     exc_group = parser.add_argument_group('Exclusions')
     excluded_slicing_arguments(exc_group,project_drs,action_type='append')
-    comp_group = parser.add_argument_group('Complex Query')
-    comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
-                                       help='List the field (or fields if repeated) found in the file' )
-    complex_slicing(comp_group,project_drs,action_type='append')
+    #comp_group = parser.add_argument_group('Complex Query')
+    #comp_group.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
+    #                                   help='List the field (or fields if repeated) found in the file' )
+    #complex_slicing(comp_group,project_drs,action_type='append')
     return
 
 def convert(subparsers,epilog,project_drs):
@@ -338,8 +340,10 @@ def certificates(subparsers,epilog,project_drs):
                                            epilog=epilog_certificates
                                          )
     parser.add_argument('username',help="Username")
-    parser.add_argument('password',help="Password")
+    #parser.add_argument('password',help="Password")
     parser.add_argument('registering_service',help="Registering service",choices=['badc','jpl'])
     return
 
+def int_list(input):
+    return [ int(item) for item in input.split(',')]
     
