@@ -70,7 +70,7 @@ class create_netCDF_pointers:
                 #netcdf_utils.replicate_netcdf_var(output,remote_data.Dataset,var_name)
                 #output.variables[var_name][:]=remote_data.Dataset.variables[var_name][:]
             output.sync()
-        except dodsError as e:
+        except remote_netcdf.dodsError as e:
             e_mod=" This is an uncommon error. It is likely to be FATAL."
             print e.value+e_mod
         remote_data.close()
@@ -100,12 +100,12 @@ class read_netCDF_pointers:
         self.variables=dict()
         return
 
-    def replicate(self,output,check_empty=False):
+    def replicate(self,output,hdf5=None,check_empty=False):
         #replicate attributes
         netcdf_utils.replicate_netcdf_file(output,self.data_root)
         #replicate and copy variables:
         for var_name in self.data_root.variables.keys():
-            netcdf_utils.replicate_and_copy_variable(output,self.data_root,var_name,check_empty=check_empty)
+            netcdf_utils.replicate_and_copy_variable(output,self.data_root,var_name,hdf5=hdf5,check_empty=check_empty)
         if 'soft_links' in self.data_root.groups.keys():
             output_grp=netcdf_utils.replicate_group(output,self.data_root,'soft_links')
             netcdf_utils.replicate_netcdf_file(output_grp,self.data_root.groups['soft_links'])
@@ -223,7 +223,7 @@ class read_netCDF_pointers:
                 dims_length.append(len(dimensions[dim]))
         #Maximum number of time step per request:
         max_request=450 #maximum request in Mb
-        max_time_steps=int(np.floor(max_request*1024*1024/(32*np.prod(dims_length))))
+        max_time_steps=max(int(np.floor(max_request*1024*1024/(32*np.prod(dims_length)))),1)
         for unique_path_id, path_id in enumerate(unique_paths_list_id):
             path_to_retrieve=self.paths_list[self.paths_id_list[path_id]]
 
@@ -261,11 +261,13 @@ class read_netCDF_pointers:
                 data_node=retrieval_utils.get_data_node(path_to_retrieve,file_type)
                 if nc_Database.is_level_name_included_and_not_excluded('data_node',self,data_node):
                     if data_node in self.queues.keys():
-                        print 'Recovering '+'/'.join(self.tree)
+                        if var_to_retrieve==self.tree[-1]:
+                            #print 'Recovering '+var_to_retrieve+' in '+path_to_retrieve
+                            print 'Recovering '+'/'.join(self.tree)
                         self.queues[data_node].put((retrieval_function,)+copy.deepcopy(args))
                     else:
                         if (isinstance(output,netCDF4.Dataset) or
                             isinstance(output,netCDF4.Group)):
-                            netcdf_utils.assign_tree(output,*getattr(retrieval_utils,retrieval_function)(args[0],args[1]))
+                            netcdf_utils.assign_tree(output,*getattr(netcdf_utils,retrieval_function)(args[0],args[1]))
         return 
 
