@@ -3,6 +3,7 @@ import numpy as np
 import math
 
 import netCDF4
+import h5py
 import datetime
 
 import hashlib
@@ -45,21 +46,29 @@ def get_year_axis(path_name):
     return year_axis, month_axis
 
 def get_date_axis(time_var):
-    if time_var.units=='day as %Y%m%d.%f':
-        date_axis=get_date_axis_absolute(time_var)
-    else:
-        date_axis=get_date_axis_relative(time_var)
-    return date_axis
-
-def get_date_axis_relative(time_var):
+    units=time_var.units
     if 'calendar' in dir(time_var):
-        date_axis = netCDF4.num2date(time_var[:],units=time_var.units,calendar=time_var.calendar)
+        calendar=time_var.calendar
     else:
-        date_axis = netCDF4.num2date(time_var[:],units=time_var.units)
+        calendar=None
+    return get_date_axis_units(time_var[:],units,calendar)
+
+def get_date_axis_units(time_axis,units,calendar):
+    if units=='day as %Y%m%d.%f':
+        date_axis=get_date_axis_absolute(time_axis)
+    else:
+        date_axis=get_date_axis_relative(time_axis,units,calendar)
     return date_axis
 
-def get_date_axis_absolute(time_var):
-    return map(convert_to_date_absolute,time_var[:])
+def get_date_axis_relative(time_axis,units,calendar):
+    if calendar!=None:
+        date_axis = netCDF4.num2date(time_axis,units=units,calendar=calendar)
+    else:
+        date_axis = netCDF4.num2date(time_axis,units=units)
+    return date_axis
+
+def get_date_axis_absolute(time_axis):
+    return map(convert_to_date_absolute,time_axis)
 
 def convert_to_date_absolute(absolute_time):
     year=int(math.floor(absolute_time/1e4))
@@ -79,51 +88,66 @@ def convert_to_date_absolute(absolute_time):
     return datetime.datetime(year,month,day,hour,minute,seconds)
 
 #def replicate_full_netcdf_recursive(output,data,options=None):
-def replicate_full_netcdf_recursive(output,data,check_empty=False):
+def replicate_full_netcdf_recursive(output,data,hdf5=None,check_empty=False):
     for var_name in data.variables.keys():
-        replicate_and_copy_variable(output,data,var_name,check_empty=check_empty)
+        replicate_and_copy_variable(output,data,var_name,hdf5=hdf5,check_empty=check_empty)
     if len(data.groups.keys())>0:
         for group in data.groups.keys():
             output_grp=replicate_group(output,data,group)
-            replicate_full_netcdf_recursive(output_grp,data.groups[group],check_empty=check_empty)
+            replicate_full_netcdf_recursive(output_grp,data.groups[group],hdf5=hdf5[group],check_empty=check_empty)
     return
 
-def replicate_and_copy_variable(output,data,var_name,datatype=None,fill_value=None,add_dim=None,chunksize=None,zlib=None,check_empty=False):
+def replicate_and_copy_variable(output,data,var_name,datatype=None,fill_value=None,add_dim=None,chunksize=None,zlib=None,hdf5=None,check_empty=False):
+
     replicate_netcdf_var(output,data,var_name,datatype=datatype,fill_value=fill_value,add_dim=add_dim,chunksize=chunksize,zlib=zlib)
-    if len(data.variables[var_name].shape)>0 and min(data.variables[var_name].shape)>0:
-        max_request=4500 #maximum request in Mb
-        max_time_steps=max(
-                        int(np.floor(max_request*1024*1024/(32*np.prod(data.variables[var_name].shape)))),
-                        1)
 
-        num_time_chunk=int(np.ceil(data.variables[var_name].shape[0]/float(max_time_steps)))
+    if len(data.variables[var_name].shape)>0:
+        variable_size=min(data.variables[var_name].shape)
+        #Use the hdf5 library to find the real size of the stored array:
+        if hdf5!=None:
+            variable_size=hdf5[var_name].size
+        if variable_size>0:
+            max_request=4500 #maximum request in Mb
+            #max_request=9000 #maximum request in Mb
+            max_time_steps=max(
+                            int(np.floor(max_request*1024*1024/(32*np.prod(data.variables[var_name].shape)))),
+                            1)
 
-        if (len(data.variables[var_name].shape)>1 and
-            num_time_chunk>1):
-            for time_chunk in range(num_time_chunk):
-                time_slice=slice(time_chunk*max_time_steps,
-                                 min((time_chunk+1)*max_time_steps,data.variables[var_name].shape[0])
-                                 ,1)
-                                 #(time_chunk+1)*max_time_steps,1)
-                #output.variables[var_name][time_slice,...]=data.variables[var_name][time_slice,...]
-                temp=data.variables[var_name][time_slice,...]
-                #Assign only if not masked everywhere:
-                #if not ('mask' in dir(temp) and temp.mask.all()):
-                #output.variables[var_name][time_slice,...]=temp
-                if not 'mask' in dir(temp) or not check_empty:
-                    output.variables[var_name][time_slice,...]=temp
-                else: 
-                    if not temp.mask.all():
+            num_time_chunk=int(np.ceil(data.variables[var_name].shape[0]/float(max_time_steps)))
+
+            if (len(data.variables[var_name].shape)>1 and
+                num_time_chunk>1):
+                for time_chunk in range(num_time_chunk):
+                    time_slice=slice(time_chunk*max_time_steps,
+                                     min((time_chunk+1)*max_time_steps,data.variables[var_name].shape[0])
+                                     ,1)
+                                     #(time_chunk+1)*max_time_steps,1)
+                    #output.variables[var_name][time_slice,...]=data.variables[var_name][time_slice,...]
+                    temp=data.variables[var_name][time_slice,...]
+                    #Assign only if not masked everywhere:
+                    #if not ('mask' in dir(temp) and temp.mask.all()):
+                    #output.variables[var_name][time_slice,...]=temp
+                    if not 'mask' in dir(temp) or not check_empty:
                         output.variables[var_name][time_slice,...]=temp
-                output.sync()
-        else:
-            #output.variables[var_name][:]=data.variables[var_name][:]
-            temp=data.variables[var_name][:]
-            if not 'mask' in dir(temp) or not check_empty:
-                output.variables[var_name][:]=temp
-            else: 
-                if not temp.mask.all():
+                    else: 
+                        #Only write the variable if it is not empty:
+                        if not temp.mask.all():
+                            output.variables[var_name][time_slice,...]=temp
+                    output.sync()
+            else:
+                #output.variables[var_name][:]=data.variables[var_name][:]
+                temp=np.reshape(data.variables[var_name][:],data.variables[var_name].shape)
+                if not 'mask' in dir(temp) or not check_empty:
+                    if output.variables[var_name].shape!=temp.shape:
+                        print data.variables[var_name].shape,  output.variables[var_name].shape,temp.shape
                     output.variables[var_name][:]=temp
+                else: 
+                    #Only write the variable if it is not empty:
+                    if not temp.mask.all():
+                        output.variables[var_name][:]=temp
+    elif len(data.variables[var_name].dimensions)==0:
+        #scalar variable:
+        output.variables[var_name][:]=data.variables[var_name][:]
     return output
 
 def replicate_group(output,data,group_name):
@@ -159,12 +183,13 @@ def replicate_netcdf_var_dimensions(output,data,var):
                 dim_var = output.createVariable(dims,data.variables[dims].dtype,(dims,))
                 dim_var[:] = data.variables[dims][:]
                 output = replicate_netcdf_var(output,data,dims)
-                if 'bounds' in output.variables[dims].ncattrs():
+                if ('bounds' in output.variables[dims].ncattrs() and
+                    output.variables[dims].getncattr('bounds') in data.variables.keys()):
                     output=replicate_netcdf_var(output,data,output.variables[dims].getncattr('bounds'))
                     output.variables[output.variables[dims].getncattr('bounds')][:]=data.variables[output.variables[dims].getncattr('bounds')][:]
             else:
                 #Create a dummy dimension variable:
-                dim_var = output.createVariable(dims,np.uint32,(dims,))
+                dim_var = output.createVariable(dims,np.float,(dims,))
                 dim_var[:]=np.arange(len(data.dimensions[dims]))
     return output
 
@@ -176,7 +201,13 @@ def replicate_netcdf_var(output,data,var,datatype=None,fill_value=None,add_dim=N
         datatype=output.createCompoundType(datatype.dtype,datatype.name)
 
     kwargs=dict()
-    kwargs['fill_value']=fill_value
+    if (fill_value==None and 
+        '_FillValue' in dir(data.variables[var]) and 
+        datatype==data.variables[var].datatype):
+            kwargs['fill_value']=data.variables[var]._FillValue
+    else:
+        kwargs['fill_value']=fill_value
+
     if zlib==None:
         if data.variables[var].filters()==None:
             kwargs['zlib']=False
@@ -201,6 +232,7 @@ def replicate_netcdf_var(output,data,var,datatype=None,fill_value=None,add_dim=N
         else:
             if data.variables[var].chunking()=='contiguous':
                 kwargs['contiguous']=True
+                kwargs['zlib']=False
             else:
                 kwargs['chunksizes']=data.variables[var].chunking()
         output.createVariable(var,datatype,dimensions,**kwargs)
@@ -313,15 +345,18 @@ def apply(options,project_drs):
     if options.script=='': return
     database=cdb_query_archive_class.SimpleTree(options,project_drs)
     #Recover the database meta data:
-    if options.field!=None:
+    if options.keep_field!=None:
         drs_to_eliminate=[field for field in database.drs.official_drs_no_version if
-                                             not field in options.field]
+                                             not field in options.keep_field]
     else:
         drs_to_eliminate=database.drs.official_drs_no_version
     vars_list=[[ var[drs_to_eliminate.index(field)] if field in drs_to_eliminate else None
                         for field in database.drs.official_drs_no_version] for var in 
                         database.list_fields_local(options,drs_to_eliminate) ]
 
+    #Always add the fixed variables to apply:
+    #vars_list=[ var for var in vars_list if var[database.drs.official_drs_no_version.index('ensemble')] !='r0i0p0' ]
+
     output_root=distributed_apply(apply_to_variable,database,options,vars_list)
     database.record_header(options,output_root)
     output_root.close()
@@ -337,12 +372,31 @@ def apply_to_variable(database,options):
     var=[getattr(options,opt) for opt in database.drs.official_drs_no_version]
     tree=zip(database.drs.official_drs_no_version,var)
 
+    if options.add_fixed:
+        #Specification for fixed vars:
+        var_fx=[ getattr(options,opt) if not opt in database.drs.var_specs+['var',] else None for opt in database.drs.official_drs_no_version]
+        var_fx=copy.copy(var)
+        var_fx[database.drs.official_drs_no_version.index('ensemble')]='r0i0p0'
+        var_fx[database.drs.official_drs_no_version.index('var')]=None
+        for opt  in database.drs.var_specs+['var',]:
+            if opt in ['time_frequency','cmor_table']:
+                var_fx[database.drs.official_drs_no_version.index(opt)]='fx'
+        tree_fx=zip(database.drs.official_drs_no_version,var_fx)
+        options_fx=copy.copy(options)
+        for opt_id,opt in enumerate(tree_fx):
+            if opt!=tree[opt_id]:
+                setattr(options_fx,opt[0],opt[1])
+
     temp_files_list=[]
     for file in files_list:
         data=netCDF4.Dataset(file,'r')
         temp_file=file+'.pid'+str(os.getpid())
         output_tmp=netCDF4.Dataset(temp_file,'w',format='NETCDF4',diskless=True,persist=True)
-        extract_netcdf_variable_recursive(output_tmp,data,tree[0],tree[1:],options,check_empty=True)
+        #extract_netcdf_variable_recursive(output_tmp,data,tree[0],tree[1:],options,check_empty=True)
+        extract_netcdf_variable_recursive(output_tmp,data,tree[0],tree[1:],options,check_empty=False)
+        if options.add_fixed:
+            #extract_netcdf_variable_recursive(output_tmp,data,tree_fx[0],tree_fx[1:],options_fx,check_empty=True)
+            extract_netcdf_variable_recursive(output_tmp,data,tree_fx[0],tree_fx[1:],options_fx,check_empty=False)
         temp_files_list.append(temp_file)
         output_tmp.close()
         data.close()
@@ -390,7 +444,10 @@ def extract_netcdf_variable_recursive(output,data,level_desc,tree,options,check_
                     netcdf_pointers.replicate(output_grp,check_empty=check_empty)
     else:
         if len(tree)>0:
-            extract_netcdf_variable_recursive(output,data.groups[group_name],tree[0],tree[1:],options,check_empty=check_empty)
+            if group_name!='':
+                extract_netcdf_variable_recursive(output,data.groups[group_name],tree[0],tree[1:],options,check_empty=check_empty)
+            else:
+                extract_netcdf_variable_recursive(output,data,tree[0],tree[1:],options,check_empty=check_empty)
         else:
             netcdf_pointers=netcdf_soft_links.read_netCDF_pointers(data.groups[group_name])
             netcdf_pointers.replicate(output,check_empty=check_empty)
@@ -430,15 +487,17 @@ def distributed_apply(function_handle,database,options,vars_list,args=tuple()):
                 temp_file_name=description[0]
                 var=description[1]
                 data=netCDF4.Dataset(temp_file_name,'r')
+                data_hdf5=h5py.File(temp_file_name,'r')
                 tree=zip(database.drs.official_drs_no_version,var)
-                replace_netcdf_variable_recursive(output_root,data,tree[0],tree[1:],check_empty=True)
+                replace_netcdf_variable_recursive(output_root,data,tree[0],tree[1:],hdf5=data_hdf5,check_empty=True)
                 data.close()
+                data_hdf5.close()
                 try:
                     os.remove(temp_file_name)
                 except OSError:
                     pass
                 output_root.sync()
-
+    
             pool.close()
             pool.join()
         else:
@@ -446,9 +505,11 @@ def distributed_apply(function_handle,database,options,vars_list,args=tuple()):
                 worker(arg)
                 temp_file_name, var=queue.get()
                 data=netCDF4.Dataset(temp_file_name,'r')
+                data_hdf5=h5py.File(temp_file_name,'r')
                 tree=zip(database.drs.official_drs_no_version,var)
-                replace_netcdf_variable_recursive(output_root,data,tree[0],tree[1:],check_empty=True)
+                replace_netcdf_variable_recursive(output_root,data,tree[0],tree[1:],hdf5=data_hdf5,check_empty=True)
                 data.close()
+                data_hdf5.close()
                 try:
                     os.remove(temp_file_name)
                 except OSError:
@@ -458,7 +519,7 @@ def distributed_apply(function_handle,database,options,vars_list,args=tuple()):
 
         return output_root
 
-def replace_netcdf_variable_recursive(output,data,level_desc,tree,check_empty=False):
+def replace_netcdf_variable_recursive(output,data,level_desc,tree,hdf5=None,check_empty=False):
     level_name=level_desc[0]
     group_name=level_desc[1]
     if group_name==None or isinstance(group_name,list):
@@ -466,18 +527,20 @@ def replace_netcdf_variable_recursive(output,data,level_desc,tree,check_empty=Fa
             output_grp=create_group(output,data,group)
             output_grp.setncattr('level_name',level_name)
             if len(tree)>0:
-                    replace_netcdf_variable_recursive(output_grp,data.groups[group],tree[0],tree[1:],check_empty=check_empty)
+                    replace_netcdf_variable_recursive(output_grp,data.groups[group],tree[0],tree[1:],hdf5=hdf5[group],check_empty=check_empty)
             else:
                 netcdf_pointers=netcdf_soft_links.read_netCDF_pointers(data.groups[group])
-                netcdf_pointers.replicate(output_grp,check_empty=check_empty)
+                netcdf_pointers.replicate(output_grp,hdf5=hdf5[group],check_empty=check_empty)
     else:
+        output_grp=create_group(output,data,group_name)
+        output_grp.setncattr('level_name',level_name)
         if len(tree)>0:
-            output_grp=create_group(output,data,group_name)
-            output_grp.setncattr('level_name',level_name)
-            replace_netcdf_variable_recursive(output_grp,data,tree[0],tree[1:],check_empty=check_empty)
+            replace_netcdf_variable_recursive(output_grp,data,tree[0],tree[1:],hdf5=hdf5,check_empty=check_empty)
         else:
             netcdf_pointers=netcdf_soft_links.read_netCDF_pointers(data)
-            netcdf_pointers.replicate(output_grp,check_empty=check_empty)
+            netcdf_pointers.replicate(output_grp,hdf5=hdf5,check_empty=check_empty)
+        #    netcdf_pointers=netcdf_soft_links.read_netCDF_pointers(data)
+        #    netcdf_pointers.replicate(output,hdf5=hdf5,check_empty=check_empty)
     return
 
     
